==> accountbook-3.0b1.tar.gz: accountbook-3.0b1/public/comparison.rb <==
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 週次・月次対比表

require "pcview.rb"
require 'journal-data.rb'

module ComparativeChart
  VIEW_RANGE = 10

  def bs_pl(week_end_ary)
    begin_date = week_end_ary[0] + 1
    end_date = week_end_ary.last
    
    bs_model = {
      :assets => [],
      :liabilities => [],
      :equity => [],
      :total_assets => [],
      :total_liabilities => [],
      :total_equity => [],
    }
    pl_model = {
      :range => [],
      :sales => [],
      :operating_costs => [],
      :other_income => [],
      :net_sales => [],
      :op_costs_total => [],
      :operating_income => [],
      :other_total => [],
      :profit => [],
    }

    # 個別項目
    @bal_ary.each {|item, ary|
      node = [nil, "assets", "liabilities", "equity",
              "sales", "operating_costs", "other_income",
              "other_income"][item.ac_type]
      raise "internal error" if !node
      
      line = {
        :item => item.name,
        :amounts => []
      }

      for i in 1 .. week_end_ary.size - 1
        if item.ac_type <= 3
          if ary[i]
            h = decimal_format(ary[i] * item.sign)
          else
            h = "-"
          end
        else
          if ary[i]
            if item.ac_type == 7 # 特別損失
              h = {:amount => decimal_format(ary[i] * item.sign * -1)}
            else
              h = {:amount => decimal_format(ary[i] * item.sign)}
            end
            b = week_end_ary[i - 1] + 1
            e = week_end_ary[i]
            h[:link] = "ledger.rb?action=view&amp;byear=#{b.year}&amp;" +
                            "bmon=#{b.mon}&amp;bday=#{b.day}&amp;" +
                            "eyear=#{e.year}&amp;emon=#{e.mon}&amp;" +
                            "eday=#{e.day}&amp;code=#{item.id}"
          else
            h = {:amount => "-"}
          end
        end
        line[:amounts] << h
      end
      if item.ac_type <= 3
        bs_model[node.intern] << line
      else
        pl_model[node.intern] << line
      end
    }

    # 貸借差額は、純利益
    line = {
      :item => "当期純利益",
      :amounts => []
    }
    for i in 1 .. week_end_ary.size - 1
      line[:amounts] << decimal_format(@total[i][1] - @total[i][2] - @total[i][3])
    end
    bs_model[:equity] << line
    
    # 合計行
    for i in 1 .. week_end_ary.size - 1
      bs_model[:total_assets] << decimal_format(@total[i][1])
      bs_model[:total_liabilities] << decimal_format(@total[i][2])
      bs_model[:total_equity] << decimal_format(@total[i][1] - @total[i][2])

      pl_model[:range] << Amrita::SanitizedString["#{format_date(week_end_ary[i - 1] + 1)}〜<br>#{format_date(week_end_ary[i])}"]
      pl_model[:net_sales] << decimal_format(@total[i][4])
      pl_model[:op_costs_total] << decimal_format(@total[i][5])
      pl_model[:operating_income] << decimal_format(@total[i][4] - @total[i][5])
      pl_model[:other_total] << decimal_format(@total[i][6] - @total[i][7])
      pl_model[:profit] << decimal_format(@total[i][4] - @total[i][5] +
                                          @total[i][6] - @total[i][7])
    end

    return [bs_model, pl_model]
  end

  def cf_val(item, ary, i)
    if item.bs?
      return (ary[i - 1] || 0) - (ary[i] || 0)
    else
      return -(ary[i] || 0)
    end
  end

  def valid?(item, ary)
    (1 .. ary.size - 1).each {|i|
      return true if cf_val(item, ary, i) != 0
    }
    return false
  end

  def cf(week_end_ary)
    cf_total = []
    cash = [0]
    # 個別項目
    for i in 1 .. week_end_ary.size - 1
      cf_total[i] = []
      cash[i] = 0
      cf_total[i][1] = @total[i][4] - @total[i][5] + @total[i][6] - @total[i][7]
    end

    model = {
      :range => [],
      :profit => [],
      :cf_operating => [],
      :cf_investing => [],
      :cf_financing => [],
      :cf_net_operating => [],
      :cf_net_investing => [],
      :cf_net_financing => [],
      :cf_net_increase => [],
      :cf_at_beginning => [],
      :cf_at_end => []
    }

    amounts = []
    @bal_ary.each {|item, ary|
      # 営業C/F
      h = {}
      amounts = []
      if item.bs?
        if item.cf_type == 1 && valid?(item, ary)
          h[:item] = item.name + " の増減"
          for i in 1 .. week_end_ary.size - 1
            val = cf_val(item, ary, i)
            cf_total[i][1] = (cf_total[i][1] || 0) + val
            amounts << (val != 0 ? decimal_format(val) : "-")
          end
        end
      else
        if item.cf_type != 1
          h[:item] = CGI.escapeHTML(item.name)
          for i in 1 .. week_end_ary.size - 1
            val = cf_val(item, ary, i)
            cf_total[i][1] = (cf_total[i][1] || 0) - val
            amounts << (val != 0 ? decimal_format(-val) : "-")
          end
        end
      end
      if h.size > 0
        h[:amount] = amounts
        model[:cf_operating] << h
      end

      # 投資・財務C/F
      if (item.cf_type == 2 || item.cf_type == 3) && valid?(item, ary)
        h = {:item => item.name + (item.bs? ? " の増減" : "")}
        amounts = []
        for i in 1 .. week_end_ary.size - 1
          val = cf_val(item, ary, i)
          cf_total[i][item.cf_type] = (cf_total[i][item.cf_type] || 0) + val
          amounts << (val != 0 ? decimal_format(val) : "-")
        end
        h[:amount] = amounts
        model[[:cf_investing, :cf_financing][item.cf_type - 2]] << h
      end

      # 現金同等物
      if item.cf_type == 4
        for i in 0 .. week_end_ary.size - 1
          cash[i] += (ary[i] || 0)
        end
      end
    }

    # 合計行
    for i in 1 .. week_end_ary.size - 1
      model[:range] << Amrita::SanitizedString["#{format_date(week_end_ary[i - 1] + 1)}〜<br>#{format_date(week_end_ary[i])}"]
      model[:profit] << decimal_format(@total[i][4] - @total[i][5] +
                                       @total[i][6] - @total[i][7])
      model[:cf_net_operating] << decimal_format(cf_total[i][1])
      model[:cf_net_investing] << decimal_format(cf_total[i][2])
      model[:cf_net_financing] << decimal_format(cf_total[i][3])
      model[:cf_net_increase] << decimal_format(cash[i] - cash[i - 1])
      model[:cf_at_beginning] << decimal_format(cash[i - 1])
      model[:cf_at_end] << decimal_format(cash[i])
    end

    return model
  end
  
  # last_endの型は，DateかArray
  def table_out(pagename, last_end)
    op_date = @app.user.user_setting.opening_date

    # 仕訳・残高データ取得
    account = Account.new(@app.user)
    weekbal = {}
    @total = []  # 科目区分（資産，負債，...）ごとの合計額
    week_end = []

    ed = get_prev_end(last_end, nil)
    raise RangeError, "開始日は#{op_date}です。この日より新しい日を指定してください。" if ed <= op_date
    VIEW_RANGE.downto(0) {
      if ed <= op_date
        week_end.unshift(op_date)
        @total.unshift([nil, 0, 0, 0, 0, 0, 0, 0])
        break
      end
      week_end.unshift(ed)
      @total.unshift([nil, 0, 0, 0, 0, 0, 0, 0])
      ed = get_prev_end(last_end, ed)
    }
    fs_hash_ary = account.get_fs_multi(week_end)
    fs_hash_ary.each_with_index {|fs_hash, week|
      fs_hash.each {|code, val|
        if val != 0
          item = AccountTitle.find_item(code, @app.user)
          weekbal[item] = [] if !weekbal[item]
          weekbal[item][week] = val
          @total[week][item.ac_type] += val * item.sign
        end
      }
    }

    model = {
      :pagename => pagename,
      :bs_date => [],
      :range => []
    }

    for i in 1 .. week_end.size - 1
      model[:bs_date] << week_end[i]
      model[:range] << {
        :b_date => week_end[i - 1] + 1,
        :e_date => week_end[i]
      }
    end

    @bal_ary = weekbal.to_a.sort {|x, y| x[0].sort <=> y[0].sort}
    bs_model, pl_model = bs_pl(week_end)
    cf_model = cf(week_end)

    model.update(bs_model)
    model.update({
      :mod_pl => Amrita::SanitizedString[mod_render("pl.div.html", pl_model)],
      :mod_cf => Amrita::SanitizedString[mod_render("cf.div.html", cf_model)]
    })
    @app.page_out("week2.html", model)
  end
end

==> accountbook-3.0.0.tar.gz: accountbook-3.0.0/public/comparison.rb <==
# -*- coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/

# 週次・月次対比表

require "pcview.rb"
require 'journal-data.rb'

module ComparativeChart
  VIEW_RANGE = 10

  def bs_pl(week_end_ary, chart_data)
    begin_date = week_end_ary[0] + 1
    end_date = week_end_ary.last
    
    bs_model = {
      :assets => [],
      :liabilities => [],
      :equity => [],
      :total_assets => [],
      :total_liabilities => [],
      :total_equity => [],
    }
    pl_model = {
      :range => [],
      :sales => [],
      :operating_costs => [],
      :other_income => [],
      :net_sales => [],
      :op_costs_total => [],
      :operating_income => [],
      :other_total => [],
      :profit => [],
    }

    # 個別項目
    @bal_ary.each {|item, ary|
      node = [nil, "assets", "liabilities", "equity",
              "sales", "operating_costs", "other_income",
              "other_income"][item.ac_type]
      raise "internal error" if !node
      
      line = {
        :item => item.name,
        :amounts => []
      }

      for i in 1 .. week_end_ary.size - 1
        if item.ac_type <= 3
          # B/S項目
          if item.id == app.user.user_setting.unappropriated_item
            # 未処分利益は特別扱い
            h = decimal_format((ary[i] || 0) * item.sign + 
                               @total[i][1] - @total[i][2] - @total[i][3])
          else
            if ary[i]
              h = decimal_format(ary[i] * item.sign)
            else
              h = "-"
            end
          end
        else
          # P/L項目
          if ary[i]
            if item.ac_type == 7 # 特別損失
              h = {:amount => decimal_format(ary[i] * item.sign * -1)}
            else
              h = {:amount => decimal_format(ary[i] * item.sign)}
            end
            b = week_end_ary[i - 1] + 1
            e = week_end_ary[i]
            h[:link] = "ledger.rb?action=view&amp;byear=#{b.year}&amp;" +
                            "bmon=#{b.mon}&amp;bday=#{b.day}&amp;" +
                            "eyear=#{e.year}&amp;emon=#{e.mon}&amp;" +
                            "eday=#{e.day}&amp;code=#{item.id}"
          else
            h = {:amount => "-"}
          end
        end
        line[:amounts] << h
      end
      if item.ac_type <= 3
        bs_model[node.intern] << line
      else
        pl_model[node.intern] << line
      end
    }

    # 合計行
    for i in 1 .. week_end_ary.size - 1
      bs_model[:total_assets] << decimal_format(@total[i][1])
      bs_model[:total_liabilities] << decimal_format(@total[i][2])
      total_equity = @total[i][1] - @total[i][2]
      bs_model[:total_equity] << decimal_format(total_equity)

      pl_model[:range] << Amrita::SanitizedString["#{format_date(week_end_ary[i - 1] + 1)}〜<br>#{format_date(week_end_ary[i])}"]
      pl_model[:net_sales] << decimal_format(@total[i][4])
      pl_model[:op_costs_total] << decimal_format(@total[i][5])
      pl_model[:operating_income] << decimal_format(@total[i][4] - @total[i][5])
      pl_model[:other_total] << decimal_format(@total[i][6] - @total[i][7])
      profit = @total[i][4] - @total[i][5] + @total[i][6] - @total[i][7]
      pl_model[:profit] << decimal_format(profit)

      # グラフデータ
      chart_data[:elements][0][:values] << profit
      chart_data[:elements][1][:values] << total_equity
    end

    return [bs_model, pl_model]
  end

  def cf_val(item, ary, i)
    if item.bs?
      return (ary[i - 1] || 0) - (ary[i] || 0)
    else
      return -(ary[i] || 0)
    end
  end

  def valid?(item, ary)
    (1 .. ary.size - 1).each {|i|
      return true if cf_val(item, ary, i) != 0
    }
    return false
  end

  def cf(week_end_ary)
    cf_total = []
    cash = [0]
    # 個別項目
    for i in 1 .. week_end_ary.size - 1
      cf_total[i] = []
      cash[i] = 0
      cf_total[i][1] = @total[i][4] - @total[i][5] + @total[i][6] - @total[i][7]
    end

    model = {
      :range => [],
      :profit => [],
      :cf_operating => [],
      :cf_investing => [],
      :cf_financing => [],
      :cf_net_operating => [],
      :cf_net_investing => [],
      :cf_net_financing => [],
      :cf_net_increase => [],
      :cf_at_beginning => [],
      :cf_at_end => []
    }

    amounts = []
    @bal_ary.each {|item, ary|
      # 営業C/F
      h = {}
      amounts = []
      if item.bs?
        if item.cf_type == 1 && valid?(item, ary)
          h[:item] = item.name + " の増減"
          for i in 1 .. week_end_ary.size - 1
            val = cf_val(item, ary, i)
            cf_total[i][1] = (cf_total[i][1] || 0) + val
            amounts << (val != 0 ? decimal_format(val) : "-")
          end
        end
      else
        if item.cf_type != 1
          h[:item] = CGI.escapeHTML(item.name)
          for i in 1 .. week_end_ary.size - 1
            val = cf_val(item, ary, i)
            cf_total[i][1] = (cf_total[i][1] || 0) - val
            amounts << (val != 0 ? decimal_format(-val) : "-")
          end
        end
      end
      if h.size > 0
        h[:amount] = amounts
        model[:cf_operating] << h
      end

      # 投資・財務C/F
      if (item.cf_type == 2 || item.cf_type == 3) && valid?(item, ary)
        h = {:item => item.name + (item.bs? ? " の増減" : "")}
        amounts = []
        for i in 1 .. week_end_ary.size - 1
          val = cf_val(item, ary, i)
          cf_total[i][item.cf_type] = (cf_total[i][item.cf_type] || 0) + val
          amounts << (val != 0 ? decimal_format(val) : "-")
        end
        h[:amount] = amounts
        model[[:cf_investing, :cf_financing][item.cf_type - 2]] << h
      end

      # 現金同等物
      if item.cf_type == 4
        for i in 0 .. week_end_ary.size - 1
          cash[i] += (ary[i] || 0)
        end
      end
    }

    # 合計行
    for i in 1 .. week_end_ary.size - 1
      model[:range] << Amrita::SanitizedString["#{format_date(week_end_ary[i - 1] + 1)}〜<br>#{format_date(week_end_ary[i])}"]
      model[:profit] << decimal_format(@total[i][4] - @total[i][5] +
                                       @total[i][6] - @total[i][7])
      model[:cf_net_operating] << decimal_format(cf_total[i][1])
      model[:cf_net_investing] << decimal_format(cf_total[i][2])
      model[:cf_net_financing] << decimal_format(cf_total[i][3])
      model[:cf_net_increase] << decimal_format(cash[i] - cash[i - 1])
      model[:cf_at_beginning] << decimal_format(cash[i - 1])
      model[:cf_at_end] << decimal_format(cash[i])
    end

    return model
  end
  
  # last_endの型は，DateかArray
  def table_out(pagename, last_end)
    op_date = @app.user.user_setting.opening_date

    # 仕訳・残高データ取得
    account = Account.new(@app.user)
    weekbal = {}
    @total = []  # 科目区分（資産，負債，...）ごとの合計額
    week_end = []

    ed = get_prev_end(last_end, nil)
    raise RangeError, "開始日は#{op_date}です。この日より新しい日を指定してください。" if ed <= op_date
    VIEW_RANGE.downto(0) {
      if ed <= op_date
        week_end.unshift(op_date)
        @total.unshift([nil, 0, 0, 0, 0, 0, 0, 0])
        break
      end
      week_end.unshift(ed)
      @total.unshift([nil, 0, 0, 0, 0, 0, 0, 0])
      ed = get_prev_end(last_end, ed)
    }
    fs_hash_ary = account.get_fs_multi(week_end)
    fs_hash_ary.each_with_index {|fs_hash, week|
      fs_hash.each {|code, val|
        if val != 0
          item = AccountTitle.find_item(code, @app.user)
          weekbal[item] = [] if !weekbal[item]
          weekbal[item][week] = val
          @total[week][item.ac_type] += val * item.sign
        end
      }
    }

    model = {
      :pagename => pagename,
      :bs_date => [],
      :range => []
    }

    for i in 1 .. week_end.size - 1
      model[:bs_date] << week_end[i]
      model[:range] << {
        :b_date => week_end[i - 1] + 1,
        :e_date => week_end[i]
      }
    end

    chart_data = {
      :elements => [
        { # 純利益
          :type => "bar", 
          :values => [],
          :text => "純利益",
        }, { # 純資産
          :type => "line",
          :values => [],
          :text => "純資産",
          :colour => "#009900"
        }
      ]
    }

    @bal_ary = weekbal.to_a.sort {|x, y| x[0].sort <=> y[0].sort}
    bs_model, pl_model = bs_pl(week_end, chart_data)
    cf_model = cf(week_end)

    # チャートを整える
    chart_data[:y_axis] = {
      :min => (chart_data[:elements][0][:values] + 
               chart_data[:elements][1][:values] + [0]).min,
      :max => (chart_data[:elements][0][:values] + 
               chart_data[:elements][1][:values] + [0]).max
    }
    chart_data[:y_axis][:steps] = (chart_data[:y_axis][:max] - 
                                   chart_data[:y_axis][:min]) / 10 + 1

    model.update(bs_model)
    model.update({
      :mod_pl => Amrita::SanitizedString[mod_render("pl.div.html", pl_model)],
      :mod_cf => Amrita::SanitizedString[mod_render("cf.div.html", cf_model)],
      :chart_data => ActiveSupport::JSON.encode(chart_data)
    })
    app.page_out("week2.html", model)
  end
end
